==> 0.0.2.tar.gz: lib/pages/comments.dart <==
import 'dart:async';
import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:timeago/timeago.dart' as time_ago;

import 'package:bizdoc/gen_l10n/app_localizations.dart';
import 'package:bizdoc/services/accounts.dart';
import 'package:bizdoc/services/session.dart';
import 'package:bizdoc/services/mailbox.dart';
import 'package:bizdoc/models/contracts.dart';
import 'package:bizdoc/models/models.dart';
import 'package:bizdoc/core/hub.dart';

/// Lists comments for a message. Listen to hub [CommentContractBase] stream.
class CommentsWidget extends StatefulWidget {
  final RecipientModel model;
  final Function onTap;
  const CommentsWidget({Key? key, required this.model, required this.onTap})
      : super(key: key);
  @override
  _CommentsWidgetState createState() => _CommentsWidgetState();
}

class _CommentsWidgetState extends State<CommentsWidget> {
  final _controller = TextEditingController();
  late Stream<CommentContractBase> _changes;
  @override
  void initState() {
    _changes = hub.events
        .where((event) =>
            event is CommentContractBase && event.id == widget.model.documentId)
        .cast();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder(
        future: _decorate(),
        builder: (context, snapshot) {
          if (snapshot.hasData) {
            return StreamBuilder(
              stream: _changes,
              builder: (context, snapshot) {
                return Column(
                  children: [
                    Expanded(
                        child: ListView(
                      children: [
                        for (var item in widget.model.comments)
                          ListTile(
                            key: Key(item.id.toString()),
                            leading: item.image != null
                                ? Image.memory(base64Decode(item.image!))
                                : null,
                            title: Text(item.text ?? ''),
                            trailing: Text(time_ago.format(item.time)),
                            subtitle: Text(item.name ?? ''),
                          ),
                      ],
                    )),
                    TextField(
                      autocorrect: true,
                      autofocus: widget.model.comments.isEmpty,
                      enableSuggestions: true,
                      controller: _controller,
                      decoration: InputDecoration(
                          hintText: AppLocalizations.of(context)!.typeHere),
                      onChanged: (_) => _typing(),
                      onSubmitted: _send,
                    )
                  ],
                );
              },
            );
          }
          return const LinearProgressIndicator();
        });
  }

  Future<dynamic> _typing() async {
    widget.onTap();
    try {
      mailbox.commentsTyping(widget.model.documentId);
    } catch (e) {}
  }

// submit comment
  Future<void> _send(String text) async {
    if (text.isNotEmpty) {
      try {
        final model =
            await mailbox.comment(widget.model.documentId, text: text);
        setState(() {
          model.name = AppLocalizations.of(context)!.you;
          widget.model.comments.add(model);
        });
        _controller.clear();
      } catch (e) {
        ScaffoldMessenger.of(context).showSnackBar(
            SnackBar(content: Text(AppLocalizations.of(context)!.err)));
      }
    }
  }

  /// add user name to items
  ///
  Future _decorate() {
    return Future.wait(widget.model.comments.map((e) async {
      if (e.userId == (session.profile.byId ?? session.profile.userId)) {
        e.name = AppLocalizations.of(context)!.you;
      } else {
        final info = await accounts.userInfo(e.userId);
        e.name = info.name;
      }
    }));
  }
}

==> 0.0.3.tar.gz: lib/pages/comments.dart <==
import 'dart:async';
import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:timeago/timeago.dart' as time_ago;

import 'package:bizdoc/gen_l10n/app_localizations.dart';
import 'package:bizdoc/services/accounts.dart';
import 'package:bizdoc/services/session.dart';
import 'package:bizdoc/services/mailbox.dart';
import 'package:bizdoc/models/contracts.dart';
import 'package:bizdoc/models/data.dart';
import 'package:bizdoc/core/hub.dart';

/// Lists comments for a message. Listen to hub [CommentContractBase] stream.
class CommentsWidget extends StatefulWidget {
  final RecipientModel model;
  final Function onTap;
  const CommentsWidget({Key? key, required this.model, required this.onTap})
      : super(key: key);
  @override
  _CommentsWidgetState createState() => _CommentsWidgetState();
}

class _CommentsWidgetState extends State<CommentsWidget> {
  final _controller = TextEditingController();
  late Stream<CommentContractBase> _changes;
  @override
  void initState() {
    _changes = hub.events
        .where((event) =>
            event is CommentContractBase && event.id == widget.model.documentId)
        .cast();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder(
        future: _decorate(),
        builder: (context, snapshot) {
          if (snapshot.hasData) {
            return StreamBuilder(
              stream: _changes,
              builder: (context, snapshot) {
                return Column(
                  children: [
                    Expanded(
                        child: ListView(
                      children: [
                        for (var item in widget.model.comments)
                          ListTile(
                            key: Key(item.id.toString()),
                            leading: item.image != null
                                ? Image.memory(base64Decode(item.image!))
                                : null,
                            title: Text(item.text ?? ''),
                            trailing: Text(time_ago.format(item.time)),
                            subtitle: Text(item.name ?? ''),
                          ),
                      ],
                    )),
                    TextField(
                      autocorrect: true,
                      autofocus: widget.model.comments.isEmpty,
                      enableSuggestions: true,
                      controller: _controller,
                      decoration: InputDecoration(
                          hintText: AppLocalizations.of(context)!.typeHere),
                      onChanged: (_) => _typing(),
                      onSubmitted: _send,
                    )
                  ],
                );
              },
            );
          }
          return const LinearProgressIndicator();
        });
  }

  Future<dynamic> _typing() async {
    widget.onTap();
    try {
      mailbox.commentsTyping(widget.model.documentId);
    } catch (e) {}
  }

// submit comment
  Future<void> _send(String text) async {
    if (text.isNotEmpty) {
      try {
        final model =
            await mailbox.comment(widget.model.documentId, text: text);
        setState(() {
          model.name = AppLocalizations.of(context)!.you;
          widget.model.comments.add(model);
        });
        _controller.clear();
      } catch (e) {
        ScaffoldMessenger.of(context).showSnackBar(
            SnackBar(content: Text(AppLocalizations.of(context)!.err)));
      }
    }
  }

  /// add user name to items
  ///
  Future _decorate() {
    return Future.wait(widget.model.comments.map((e) async {
      if (e.userId == (session.profile.byId ?? session.profile.userId)) {
        e.name = AppLocalizations.of(context)!.you;
      } else {
        final info = await accounts.find(e.userId);
        e.name = info.name;
      }
    }));
  }
}
